/************************************************************************

 The contents of this file are subject to the Mozilla Public License
 Version 1.1 (the "License"); you may not use this file except in	
 compliance with the License. You may obtain a copy of the License at
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS"
 basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 License for the specific language governing rights and limitations
 under the License.

 The Original Code is Isearch.

 The Initial Developer of the Original Code is Clearinghouse for
 Networked Information Discovery and Retrieval.

 Portions created by Netscape Communications are Copyright (C) 2001. 
 All Rights Reserved.

 Portions created by the Clearinghouse for Networked Information
 Discovery and Retrieval are Copyright (C) 1994. All Rights Reserved.

************************************************************************/

#include "idb.h"
#include "mdthashtable.h"

static FILE* GlobalFcFp;
static GDT_BOOLEAN GlobalWrongEndian;
MDTHASHTABLE ht;

IDB::IDB(const STRING& NewPathName, const STRING& NewFileName) {
  Initialize(NewPathName, NewFileName);
}

void
IDB::Initialize(const STRING& NewPathName, const STRING& NewFileName) {

  STRING fn;

  Title = NewFileName;

  GlobalDocType = DOC_TYPE_INVALID;

  // initialize the MDT hashtable
  fn = NewPathName;
  if (fn != "") {
    fn.Cat("/");
  }
  fn.Cat(NewFileName);

  ht.Init((char*)fn.Buffer);

  DebugMode = 0;
  DebugSkip = 0;
  TotalRecordsQueued = 0;
  DbPathName = NewPathName;
  AddTrailingSlash(&DbPathName);
  DbFileName = NewFileName;
  RemovePath(&DbFileName);

  // initialize restriced dir name
  RestrictDir = "";

  TopLevelRestrictDir = 0;

  // restrict to non-adult results by default
  RestrictAdult = 1;

  SetWrongEndian();

  // Create INDEX
  STRING IndexFN;
  ComposeDbFn(&IndexFN, DbExtIndex);
  MainIndex = new INDEX(this, IndexFN);

  // Create and load MDT
  STRING MDTFN;
  ComposeDbFn(&MDTFN, DbExtMdt);
  STRING FileStem;
  GetDbFileStem(&FileStem);
  MainMdt = new MDT((char *)FileStem.Buffer, IsWrongEndian());

  DocTypeReg = new DTREG(this);

  if (MainMdt->MdtFd < 0) {
    // failed to open the MDT database file
    return;
  }

  GlobalDocType = GetGlobalDocType();

}

void
IDB::SetRestrictDir(const STRING& DirName) {

    DOCTYPE *DocTypePtr = DocTypeReg->GetDocTypePtr(GetGlobalDocType());

    // setting this restricts all searches to the particular
    // set of sub-directories specified
    RestrictDir = DirName;

    TopLevelRestrictDir = DocTypePtr->TopLevelCat(DirName.Buffer);
}

char *
IDB::GetRestrictDir() {

  // return a pointer to the restricted dir name (static pointer, do not free!)
  return ((char *)RestrictDir.Buffer);

}

void
IDB::SetMaxRecordLength(int MaxLength) {

  MaxRecordLength = MaxLength;
}

int
IDB::GetMaxRecordLength() {

  return (MaxRecordLength);

}

void
IDB::SetTopLevelRestrictDir(unsigned int TopLevelDirs) {

  // The bitmask of all the top level categories to include
  TopLevelRestrictDir = TopLevelDirs;
}

unsigned int
IDB::GetTopLevelRestrictDir() {

  // return a pointer to the restricted dir name (static pointer, do not free!)
  return (TopLevelRestrictDir);

}

void
IDB::SetRestrictAdult(int i) {

  RestrictAdult = i;
}

int
IDB::GetRestrictAdult() {

  return (RestrictAdult);

}

void 
IDB::SetWrongEndian() {

    // XXX - we should have a test like this:
    //   WrongEndian = (IsDbBigEndian() != IsBigEndian()) ? GDT_TRUE : GDT_FALSE;

    WrongEndian = GDT_FALSE;
}

SIZE_T 
IDB::GpFwrite(GPTYPE* Ptr, SIZE_T Size, SIZE_T NumElements, 
	      FILE* Stream) const {
    if (IsWrongEndian()) {
	SIZE_T y;
	for (y=0; y<((Size*NumElements)/sizeof(GPTYPE)); y++) {
	    GpSwab(Ptr + y);
	}
    }
    SIZE_T val;
    val = fwrite((char*)Ptr, Size, NumElements, Stream);
    if (val < NumElements) {
	printf("ERROR: Can't Complete Write!\n");
    }
    return val;
}


SIZE_T 
IDB::GpFread(GPTYPE* Ptr, SIZE_T Size, SIZE_T NumElements, 
	     FILE* Stream) const {
  SIZE_T x = fread((char*)Ptr, Size, NumElements, Stream);
  if ( (x) && (IsWrongEndian()) ) {
    SIZE_T y;
    for (y=0; y<((Size *x)/sizeof(GPTYPE)); y++) {
      GpSwab(Ptr + y);
    }
  }
  return x;
}

GDT_BOOLEAN 
IDB::IsDbCompatible() const {
    // XXX - we sould do some tests here
    return GDT_TRUE;
}


void 
IDB::ComposeDbFn(STRING *StringBuffer, const CHR *Suffix) const {
    GetDbFileStem(StringBuffer);
    StringBuffer->Cat(Suffix);
}


void 
IDB::GetDbFileStem(STRING *StringBuffer) const {
    *StringBuffer = DbPathName;
    StringBuffer->Cat(DbFileName);
}


DOCTYPE*
IDB::GetDocTypePtr(unsigned char DocType) const {
    DOCTYPE *DoctypePtr = DocTypeReg->GetDocTypePtr(DocType);
    if (DoctypePtr) {
	return DoctypePtr;
    } else {
	return DocTypeReg->GetDocTypePtr(0);
    }
}


GDT_BOOLEAN 
IDB::ValidateDocType(STRING DocTypeString) const {
    unsigned char tmp;
    DTREG dtreg(0);
    tmp = dtreg.GetDocTypeFromString(DocTypeString);
    if (tmp == DOC_TYPE_INVALID)
	return GDT_FALSE;
    else
	return (DocTypeReg->GetDocTypePtr(tmp) != NULL) ? GDT_TRUE : GDT_FALSE;
}

GDT_BOOLEAN 
IDB::ValidateDocType(unsigned char DocType) const {
    return (DocTypeReg->GetDocTypePtr(DocType) != NULL)? GDT_TRUE : GDT_FALSE;
}


IRSET*
IDB::Search(search_query *SearchQuery) {
  if (!IsDbCompatible()) {
    return (new IRSET(this));
  }
  IRSET *RsetPtr;

  RsetPtr = MainIndex->Search(SearchQuery);

  return RsetPtr;
}


char *
IDB::GetDbVersionNumber() const {
    return(VERS);
}


//
// Read in file list, Parse the files, generate an unsorted list of records, call AddRecordList()
//

void 
IDB::Index(FILE *filelist) {

    if (!IsDbCompatible()) {
      return;
    }

    // read files one by one from the file list
    // and add them

    IndexingStatus(IndexingStatusParsingFiles, 0, 0);

    RECORD Record;
    
    // Check whether we need to set the Global DocType
    GDT_BOOLEAN SetGDocType;
    
    if (GlobalDocType == DOC_TYPE_INVALID) {
      printf("ERROR: no doctype set.\n");
    }
    
    MainIndex->AddRecordList(filelist);

    TotalRecordsQueued = 0;

}


#if 0
INT 
IDB::IsStopWord(CHR* WordStart, INT WordMaximum) const {
  return ( MainIndex->IsStopWord(WordStart, WordMaximum) );
}
#endif
   

GPTYPE 
IDB::ParseWords(unsigned char DocType, 
		unsigned char* DataBuffer, 
		int DataLength,
		int DataOffset, 
		unsigned int FileId,
		int *fdList)
{
  // Redirect the call to this method to the appropriate doctype.
  PDOCTYPE DocTypePtr;

  DocTypePtr = GetDocTypePtr(DocType);
  return ( DocTypePtr->ParseWords(DataBuffer, 
				  DataLength, 
				  DataOffset, 
				  FileId,
				  fdList) );
}
 
void 
IDB::SetDocumentInfo(const INT Index, const RECORD& Record) {
  MDTREC Mdtrec;
  MainMdt->GetEntry(Index, &Mdtrec);

  Mdtrec.fileName = Record.fileName;
  Mdtrec.pathName = Record.pathName;
  Mdtrec.SetLocalRecordStart(Record.GetRecordStart());
  Mdtrec.SetLocalRecordEnd(Record.GetRecordEnd());

  Mdtrec.SetDocumentType(Record.GetDocumentType());

  MainMdt->SetEntry(Index, Mdtrec);
}


void 
IDB::GetDocumentInfo(const INT Index, RECORD *RecordBuffer) const {
  MDTREC Mdtrec;
  MainMdt->GetEntry(Index, &Mdtrec);
  RECORD Record;

  Record.fileName = Mdtrec.fileName;
  Record.pathName = Mdtrec.pathName;
  Record.SetRecordStart(Mdtrec.GetLocalRecordStart());
  Record.SetRecordEnd(Mdtrec.GetLocalRecordEnd());

  Record.SetDocumentType(Mdtrec.GetDocumentType());

  *RecordBuffer = Record;
}


GDT_BOOLEAN 
IDB::GetDocumentDeleted(const INT Index) const {
  MDTREC Mdtrec;
  MainMdt->GetEntry(Index, &Mdtrec);
  return Mdtrec.GetDeleted();
}


SIZE_T 
IDB::CleanupDb() {
    // nothing to be done anymore
    return (0);
}

void 
IDB::SetGlobalDocType(unsigned char NewGlobalDocType) {
  GlobalDocType = NewGlobalDocType;
}


unsigned char
IDB::GetGlobalDocType() {

  if (GlobalDocType == DOC_TYPE_INVALID) {
      if (MainMdt->GetTotalEntries() > 0) {
          MDTREC Mdtrec;
	  MainMdt->GetEntry(1, &Mdtrec);
	  GlobalDocType = Mdtrec.GetDocumentType();
      }
  }

  return GlobalDocType;
}


IDB::~IDB() {
  if (DebugMode) {
    MainMdt->Dump();
  }

  if (MainIndex) {
    delete MainIndex;
  }
  if (MainMdt) {
    delete MainMdt;
  }
  delete DocTypeReg;
}


